package com.zanxiang.game.module.manage.service.impl;

import com.alibaba.nacos.common.utils.CollectionUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.game.module.manage.pojo.dto.GameDTO;
import com.zanxiang.game.module.manage.pojo.params.GameGiftPackLinkLogListParam;
import com.zanxiang.game.module.manage.pojo.vo.GameGiftPackLinkLogVO;
import com.zanxiang.game.module.manage.service.IGameGiftPackLinkLogService;
import com.zanxiang.game.module.manage.service.IGameGiftPackLinkService;
import com.zanxiang.game.module.manage.service.IGameServerService;
import com.zanxiang.game.module.manage.service.IGameService;
import com.zanxiang.game.module.mybatis.entity.GameGiftPackLink;
import com.zanxiang.game.module.mybatis.entity.GameGiftPackLinkLog;
import com.zanxiang.game.module.mybatis.entity.GameServer;
import com.zanxiang.game.module.mybatis.mapper.GameGiftPackLinkLogMapper;
import com.zanxiang.module.util.bean.BeanUtil;
import com.zanxiang.module.util.exception.BaseException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.net.URLDecoder;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author : lingfeng
 * @time : 2022-06-22
 * @description : 游戏礼包码链接访问日志
 */
@Slf4j
@Service
public class GameGiftPackLinkLogServiceImpl extends ServiceImpl<GameGiftPackLinkLogMapper, GameGiftPackLinkLog> implements IGameGiftPackLinkLogService {

    @Autowired
    private IGameService gameService;

    @Autowired
    private IGameServerService gameServerService;

    @Autowired
    private IGameGiftPackLinkService gameGiftPackLinkService;

    @Override
    public IPage<GameGiftPackLinkLogVO> list(GameGiftPackLinkLogListParam param) {
        return page(param.toPage(), new QueryWrapper<GameGiftPackLinkLog>().lambda()
                .eq(GameGiftPackLinkLog::getLinkId, param.getLinkId())
                .ge(param.getCreateBeginTime() != null, GameGiftPackLinkLog::getCreateTime, param.getCreateBeginTime() == null ? null : LocalDateTime.of(param.getCreateBeginTime(), LocalTime.MIN))
                .le(param.getCreateEndTime() != null, GameGiftPackLinkLog::getCreateTime, param.getCreateEndTime() == null ? null : LocalDateTime.of(param.getCreateEndTime(), LocalTime.MAX))
                .orderByDesc(GameGiftPackLinkLog::getCreateTime)
        ).convert(log -> BeanUtil.copy(log, GameGiftPackLinkLogVO.class));
    }

    @Override
    public Map<String, String> getSourceGameServer(Long linkLogId) {
        //查询访问日志
        GameGiftPackLinkLog gameGiftPackLinkLog = super.getById(linkLogId);
        assert gameGiftPackLinkLog != null : "参数错误, 链接访问日志信息不存在";
        //查询游戏信息
        GameDTO gameDTO = gameService.getById(gameGiftPackLinkLog.getGameId());
        assert gameDTO != null : "参数错误, 游戏信息不存在";
        //查询区服信息
        return gameServerService.list(new LambdaQueryWrapper<GameServer>()
                .eq(GameServer::getGameId, gameDTO.getSuperGameId())
                .eq(GameServer::getIsSourceServer, Boolean.TRUE)
        ).stream().collect(Collectors.toMap(GameServer::getServerId, GameServer::getServerName));
    }

    @Override
    public Long linkVisitLogPush(String url) {
        Map<String, String> urlParameter = this.getUrlParameter(url);
        String gameId = urlParameter.get("gameId");
        String codeType = urlParameter.get("codeType");
        if (CollectionUtils.isMapEmpty(urlParameter) || StringUtils.isAnyEmpty(gameId, codeType)) {
            throw new BaseException("非法链接请求, 链接参数解析异常");
        }
        //查询链接信息
        GameGiftPackLink gameGiftPackLink = gameGiftPackLinkService.getOne(new LambdaQueryWrapper<GameGiftPackLink>()
                .eq(GameGiftPackLink::getGameId, Long.valueOf(gameId))
                .eq(GameGiftPackLink::getCodeType, codeType));
        assert gameGiftPackLink != null : "非法链接请求, 链接信息不存在";
        //构造
        GameGiftPackLinkLog gameGiftPackLinkLog = this.transform(gameGiftPackLink, urlParameter);
        super.save(gameGiftPackLinkLog);
        return gameGiftPackLinkLog.getId();
    }

    private GameGiftPackLinkLog transform(GameGiftPackLink gameGiftPackLink, Map<String, String> urlParameter) {
        return GameGiftPackLinkLog.builder()
                .linkId(gameGiftPackLink.getId())
                .linkUrl(urlParameter.get("linkUrl"))
                .gameId(gameGiftPackLink.getGameId())
                .corpId(urlParameter.get("corpId"))
                .corpUserId(urlParameter.get("corpUserId"))
                .externalUserId(urlParameter.get("externalUserId"))
                .createTime(LocalDateTime.now())
                .build();
    }

    private Map<String, String> getUrlParameter(String url) {
        //参数判断
        if (Strings.isBlank(url)) {
            return Collections.emptyMap();
        }
        //没有拼接参数
        if (url.indexOf('?') == -1) {
            return Collections.singletonMap("url", url);
        }
        //参数map
        Map<String, String> map = new HashMap<>(6);
        map.put("linkUrl", url);
        try {
            final String charset = "utf-8";
            url = URLDecoder.decode(url, charset);
            //解析参数
            final String contents = url.substring(url.indexOf('?') + 1);
            String[] keyValues = contents.split("&");
            for (String keyValue : keyValues) {
                String key = keyValue.substring(0, keyValue.indexOf("="));
                String value = keyValue.substring(keyValue.indexOf("=") + 1);
                map.put(key, value);
            }
        } catch (Exception e) {
            log.error("url : {}, 链接参数解析异常", url);
            throw new BaseException("链接参数解析异常");
        }
        return map;
    }
}




